@@ -41,14 +41,12 @@ import (
 	"k8s.io/apimachinery/pkg/util/strategicpatch"
 	"k8s.io/client-go/discovery"
 	"k8s.io/client-go/dynamic"
-	"k8s.io/client-go/dynamic/dynamicinformer"
 	"k8s.io/client-go/informers"
 	"k8s.io/client-go/kubernetes"
 	"k8s.io/client-go/kubernetes/fake"
 	restclient "k8s.io/client-go/rest"
 	"k8s.io/client-go/util/workqueue"
 	"k8s.io/kubernetes/pkg/api/legacyscheme"
-	"k8s.io/kubernetes/pkg/controller"
 )
 
 type testRESTMapper struct {
@@ -74,15 +72,13 @@ func TestGarbageCollectorConstruction(t *testing.T) {
 		{Group: "tpr.io", Version: "v1", Resource: "unknown"}: {},
 	}
 	client := fake.NewSimpleClientset()
-
 	sharedInformers := informers.NewSharedInformerFactory(client, 0)
-	dynamicInformers := dynamicinformer.NewDynamicSharedInformerFactory(dynamicClient, 0)
+
 	// No monitor will be constructed for the non-core resource, but the GC
 	// construction will not fail.
 	alwaysStarted := make(chan struct{})
 	close(alwaysStarted)
-	gc, err := NewGarbageCollector(dynamicClient, rm, twoResources, map[schema.GroupResource]struct{}{},
-		controller.NewInformerFactory(sharedInformers, dynamicInformers), alwaysStarted)
+	gc, err := NewGarbageCollector(dynamicClient, rm, twoResources, map[schema.GroupResource]struct{}{}, sharedInformers, alwaysStarted)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -433,6 +429,36 @@ func TestDependentsRace(t *testing.T) {
 	}()
 }
 
+// test the list and watch functions correctly converts the ListOptions
+func TestGCListWatcher(t *testing.T) {
+	testHandler := &fakeActionHandler{}
+	srv, clientConfig := testServerAndClientConfig(testHandler.ServeHTTP)
+	defer srv.Close()
+	podResource := schema.GroupVersionResource{Version: "v1", Resource: "pods"}
+	dynamicClient, err := dynamic.NewForConfig(clientConfig)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	lw := listWatcher(dynamicClient, podResource)
+	lw.DisableChunking = true
+	if _, err := lw.Watch(metav1.ListOptions{ResourceVersion: "1"}); err != nil {
+		t.Fatal(err)
+	}
+	if _, err := lw.List(metav1.ListOptions{ResourceVersion: "1"}); err != nil {
+		t.Fatal(err)
+	}
+	if e, a := 2, len(testHandler.actions); e != a {
+		t.Errorf("expect %d requests, got %d", e, a)
+	}
+	if e, a := "resourceVersion=1&watch=true", testHandler.actions[0].query; e != a {
+		t.Errorf("expect %s, got %s", e, a)
+	}
+	if e, a := "resourceVersion=1", testHandler.actions[1].query; e != a {
+		t.Errorf("expect %s, got %s", e, a)
+	}
+}
+
 func podToGCNode(pod *v1.Pod) *node {
 	return &node{
 		identity: objectReference{
@@ -830,23 +856,10 @@ func TestGarbageCollectorSync(t *testing.T) {
 	stopCh := make(chan struct{})
 	defer close(stopCh)
 	go gc.Run(1, stopCh)
-	// The pseudo-code of GarbageCollector.Sync():
-	// GarbageCollector.Sync(client, period, stopCh):
-	//    wait.Until() loops with `period` until the `stopCh` is closed :
-	//        wait.PollImmediateUntil() loops with 100ms (hardcode) util the `stopCh` is closed:
-	//            GetDeletableResources()
-	//            gc.resyncMonitors()
-	//            controller.WaitForCacheSync() loops with `syncedPollPeriod` (hardcoded to 100ms), until either its stop channel is closed after `period`, or all caches synced.
-	//
-	// Setting the period to 200ms allows the WaitForCacheSync() to check
-	// for cache sync ~2 times in every wait.PollImmediateUntil() loop.
-	//
-	// The 1s sleep in the test allows GetDelableResources and
-	// gc.resyncMoitors to run ~5 times to ensure the changes to the
-	// fakeDiscoveryClient are picked up.
-	go gc.Sync(fakeDiscoveryClient, 200*time.Millisecond, stopCh)
+	go gc.Sync(fakeDiscoveryClient, 10*time.Millisecond, stopCh)
 
 	// Wait until the sync discovers the initial resources
+	fmt.Printf("Test output")
 	time.Sleep(1 * time.Second)
 
 	err = expectSyncNotBlocked(fakeDiscoveryClient, &gc.workerLock)
@@ -921,15 +934,10 @@ func (_ *fakeServerResources) ServerResourcesForGroupVersion(groupVersion string
 	return nil, nil
 }
 
-// Deprecated: use ServerGroupsAndResources instead.
 func (_ *fakeServerResources) ServerResources() ([]*metav1.APIResourceList, error) {
 	return nil, nil
 }
 
-func (_ *fakeServerResources) ServerGroupsAndResources() ([]*metav1.APIGroup, []*metav1.APIResourceList, error) {
-	return nil, nil, nil
-}
-
 func (f *fakeServerResources) ServerPreferredResources() ([]*metav1.APIResourceList, error) {
 	f.Lock.Lock()
 	defer f.Lock.Unlock()
